/*
 * Copyright (c) 2005, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: Agent.java 3145 2006-08-31 03:21:38Z jmettraux $
 */

//
// Agent.java
//
// jmettraux@openwfe.org
//
// generated with 
// jtmpl 1.0.04 31.10.2002 John Mettraux (jmettraux@openwfe.org)
//

package openwfe.org.apre;

import openwfe.org.ApplicationContext;
import openwfe.org.engine.workitem.InFlowWorkItem;


/**
 * An agent may be implemented in jython or in java
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Date: 2006-08-31 05:21:38 +0200 (Thu, 31 Aug 2006) $
 * <br>$Id: Agent.java 3145 2006-08-31 03:21:38Z jmettraux $ </font>
 *
 * @author jmettraux@openwfe.org
 */
public interface Agent
{

    /**
     * The agent should upon terminating its work set the field of its 
     * workitem whose name may be found in this constant to 0 in case of
     * success (in case of failure, to any other integer value).
     */
    public final static String AGENT_RESULT 
        = "__agent_result__";

    /**
     * This parameter is used to tell the Apre that it should set up
     * a classloader for this agent (reloads it each time) and that the
     * corresponding value is the CLASSPATH for this agent.
     * If this parameter is not set, the Apre will look for the agent in
     * it's own CLASSPATH (no auto reload possible then).
     */
    public final static String P_AGENT_PATH
        = "agentPath";

    public void init 
        (String agentName,
         ApplicationContext context, 
         java.util.Map agentParams)
    throws 
        ApreException;

    public String getName ();

    public InFlowWorkItem use (InFlowWorkItem wi)
        throws ApreException;

    /**
     * This method is used by agents to set the value of 
     * the field '__agent_result__' in the handled workitem.
     * If this field's value is set to 0, it means that the agent was 
     * successful.
     * Other integers mean failure, their respective value are workflow 
     * relevant, you could use these to route the workitem based on
     * agent success or failure.
     */
    public void setAgentResult (InFlowWorkItem wi, int returnCode);

}
