/*
 * Copyright (c) 2001-2006, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ApreConsumer.java 3145 2006-08-31 03:21:38Z jmettraux $
 */

//
// ApreConsumer.java
//
// jmettraux@openwfe.org
//
// generated with 
// jtmpl 1.0.04 20.11.2001 John Mettraux (jmettraux@openwfe.org)
//

package openwfe.org.apre;

import openwfe.org.ServiceException;
import openwfe.org.OpenWfeException;
import openwfe.org.ApplicationContext;
import openwfe.org.xml.XmlUtils;
import openwfe.org.engine.listen.AbstractWorkItemConsumer;
import openwfe.org.engine.workitem.WorkItem;
import openwfe.org.engine.workitem.LaunchItem;
import openwfe.org.engine.workitem.InFlowWorkItem;
import openwfe.org.engine.workitem.Attribute;
import openwfe.org.engine.workitem.AttributeUtils;
import openwfe.org.engine.workitem.StringAttribute;
import openwfe.org.engine.workitem.IntegerAttribute;
import openwfe.org.engine.dispatch.DispatchingException;
import openwfe.org.engine.expressions.ReplyException;
import openwfe.org.engine.participants.Participant;
import openwfe.org.engine.participants.ParticipantMap;


/**
 * The ApreConsumer feeds incoming workitems to a router in the APRE 
 * configuration. Workitems usually arrive in the APRE through a listener.
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Date: 2006-08-31 05:21:38 +0200 (Thu, 31 Aug 2006) $
 * <br>$Id: ApreConsumer.java 3145 2006-08-31 03:21:38Z jmettraux $ </font>
 *
 * @author jmettraux@openwfe.org
 */
public class ApreConsumer

    extends AbstractWorkItemConsumer

{

    private final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(ApreConsumer.class.getName());

    //
    // CONSTANTS (definitions)

    //
    // FIELDS

    //
    // CONSTRUCTORS

    //
    // METHODS

    /**
     * This method looks at the wi and either reply to the corresponding
     * ParticipantExpression either launch a new wf.
     */
    public Object use (final WorkItem wi)
        throws OpenWfeException
    {
        //
        // should handle CancelItem instances too !
        
        if ( ! (wi instanceof InFlowWorkItem))
        {
            log.debug
                ("use() received wi of class "+wi.getClass().getName()+
                 ". Not handled.");
        
            //
            // (engine gets replied to)

            return null;
        }

        //
        // clean agent result

        wi.getAttributes().remove(new StringAttribute(Agent.AGENT_RESULT));

        //
        // handle

        (new UsageThread((InFlowWorkItem)wi)).start();

        //
        // flow of operations continues
        // (engine gets replied to)
        
        return null;
    }

    /**
     * Looks up the 'router' from the application context.
     */
    public static Router lookupRouter (final ApplicationContext context)
    {
        //return (Router)ApplicationContext.lookup
        //    (context, "router", "ApreContext.router");
        return (Router)context.lookup("router");
    }

    //
    // INNER CLASSES

    class UsageThread extends Thread
    {
        InFlowWorkItem item = null;

        public UsageThread (InFlowWorkItem item)
        {
            this.item = item;
        }

        public void run ()
        {
            try
            {

                InFlowWorkItem result = this.item;

                //
                // determine agent and make it work

                final Router router = 
                    lookupRouter(ApreConsumer.this.getContext());

                log.debug
                    ("UsageThread.run() participantName >"+
                     item.getParticipantName()+"<");

                Agent agent = router
                    .determineAgent(item.getParticipantName());

                if (agent == null)
                {
                    result.addHistoryItem
                        ("apre",
                         "agent '"+item.getParticipantName()+"' doesn't exist");
                    log.debug
                        ("agent '"+item.getParticipantName()+"' doesn't exist");
                }
                else
                {
                    try
                    {
                        result = agent.use(this.item);

                        log.debug
                            ("Agent '"+agent.getName()+
                             "' invoked successfully");

                        //
                        // make sure there is an agent return code in the
                        // workitem

                        enforceReturnCode(this.item, 0);
                    }
                    catch (final Throwable t)
                    {
                        result.addHistoryItem
                            ("agent::"+agent.getName(),
                             ""+t);
                        log.debug
                            ("Agent '"+agent.getName()+"' invocation failed", 
                             t);

                        //
                        // set return code to 1 (general error)

                        enforceReturnCode(this.item, 1);
                    }
                }

                //
                // forward workitem
                
                final ParticipantMap pMap = openwfe.org.engine.Definitions
                    .getParticipantMap(ApreConsumer.this.getContext());

                String engineId = item.getLastExpressionId().getEngineId();

                Participant participant = pMap.get(engineId);

                if (participant == null)
                {
                    throw new DispatchingException
                        ("Cannot send back the work item to unknown engine '"+
                         engineId+"'");
                }

                participant.dispatch(getContext(), item);
            }
            catch (Exception e)
            {
                log.warn("Workitem usage failed. Workitem lost.", e);
            }
        }
    }

    /**
     * Makes sure the workitem just handled by an agent contains an
     * agent return code.
     */
    protected void enforceReturnCode (InFlowWorkItem wi, int returnCode)
    {
        Attribute currentReturnCode = 
            (Attribute)wi.getAttribute(Agent.AGENT_RESULT);

        int iCurrentReturnCode = 0;
        if (currentReturnCode != null)
        {
            try
            {
                iCurrentReturnCode = Integer
                    .parseInt(currentReturnCode.toString());
            }
            catch (final NumberFormatException e)
            {
                //
                // discard return code

                currentReturnCode = null;
            }
        }

        if (returnCode != 0)
            //
            // error code
        {
            if (currentReturnCode != null &&
                iCurrentReturnCode != 0)
                //
                // error return code already set
            {
                return;
            }
            wi.getAttributes().put
                (Agent.AGENT_RESULT, new IntegerAttribute(returnCode));
            return;
        }

        //
        // success code
        
        if (currentReturnCode != null) return;
            // do not override potiental error code

        wi.getAttributes().put(Agent.AGENT_RESULT, new IntegerAttribute(0));
    }

    //
    // METHODS from Service

    public org.jdom.Element getStatus ()
    {
        final org.jdom.Element result = new org.jdom.Element(getName());

        result.addContent(XmlUtils.getClassElt(this));
        result.addContent(XmlUtils.getRevisionElt("$Id: ApreConsumer.java 3145 2006-08-31 03:21:38Z jmettraux $"));

        return result;
    }

}
