/*
 * Copyright (c) 2001-2006, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: BasicRouter.java 3145 2006-08-31 03:21:38Z jmettraux $
 */

//
// BasicRouter.java
//
// jmettraux@openwfe.org
//
// generated with 
// jtmpl 1.0.04 31.10.2002 John Mettraux (jmettraux@openwfe.org)
//

package openwfe.org.apre.impl;

import openwfe.org.FileUtils;
import openwfe.org.Parameters;
import openwfe.org.ServiceException;
import openwfe.org.ApplicationContext;
import openwfe.org.Parameters;
import openwfe.org.xml.XmlUtils;
import openwfe.org.apre.Agent;
import openwfe.org.apre.Router;
import openwfe.org.apre.AgentBuilder;
import openwfe.org.apre.ApreException;
import openwfe.org.engine.Definitions;


/**
 * A basic router, that fetches its info from an xml config file.
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Date: 2006-08-31 05:21:38 +0200 (Thu, 31 Aug 2006) $
 * <br>$Id: BasicRouter.java 3145 2006-08-31 03:21:38Z jmettraux $ </font>
 *
 * @author john.mettraux@openwfe.org
 */
public class BasicRouter

    extends Router

{

    private final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(BasicRouter.class.getName());

    //
    // INNER CLASSES

    abstract class Entry
    {
        /*
        java.util.List matches = null;

        public Entry (final java.util.List matches)
        {
            this.matches = matches;
        }

        public java.util.List getMatchList ()
        {
            return this.matches;
        }
        */

        String agentName = null;

        public Entry (final String agentName)
        {
            this.agentName = agentName;
        }

        public boolean matches (final String participantName)
        {
            if (log.isDebugEnabled())
                log.debug("matches() participantName >"+participantName+"<");

            /*
            java.util.Iterator it = matches.iterator();
            while (it.hasNext())
            {
                String pattern = (String)it.next();
                if (participantName.matches(pattern)) return true;
            }

            return false;
            */

            return participantName.matches(this.agentName);
        }

        public abstract Agent getAgent ()
            throws ApreException;
    }

    class InstantiateEachTimeEntry
        extends Entry
    {
        String agentBuilder = null;
        String agentName = null;
        String agentClass = null;
        java.util.Map params = null;
        
        public InstantiateEachTimeEntry 
            (final String agentBuilder, 
             final String agentName, 
             final String agentClass, 
             final java.util.Map params)
             //final java.util.List matches)
        {
            //super(matches);
            super(agentName);

            this.agentBuilder = agentBuilder;
            this.agentName = agentName;
            this.agentClass = agentClass;
            this.params = params;
        }

        public Agent getAgent ()
            throws ApreException
        {
            log.debug("instantiating agent (each time)");

            return instantiateAgent
                (this.agentBuilder, 
                 this.agentName, 
                 this.agentClass, 
                 this.params);
        }
    }

    class InstantiateOnceEntry
        extends Entry
    {
        Agent agent = null;

        public InstantiateOnceEntry 
            //(final Agent agent, final java.util.List matches)
            (final Agent agent)
        {
            //super(matches);
            super(agent.getName());

            this.agent = agent;
        }

        public Agent getAgent ()
            throws ApreException
        {
            log.debug("returning instantiated agent (once)");

            return this.agent;
        }
    }

    //
    // CONSTANTS & co

    /**
     * The BasicRouter service loads its automatic-participant descriptions
     * by reading a file pointed at by the parameter named 'agentFile' or
     * 'apFile'.
     */
    public final static String P_AGENT_FILE = "agentFile";

    /**
     * The BasicRouter service loads its automatic-participant descriptions
     * by reading a file pointed at by the parameter named 'agentFile' or
     * 'apFile'.
     */
    public final static String P_AP_FILE = "apFile";

    public final static String NAME = "name";
    public final static String REGEX = "regex";
    public final static String CLASS = "class";
    public final static String AGENT = "agent";
    public final static String AP = "automatic-participant";
    public final static String BUILDER = "builder";

    public final static String INSTANTIATE = "instantiate";

    public final static String INSTANTIATE_DEFAULT_VALUE = "once";

    //
    // FIELDS

    private java.util.List entries = null;

    //
    // CONSTRUCTORS

    public void init 
        (final String serviceName, 
         final ApplicationContext context, 
         final java.util.Map serviceParams)
    throws 
        ServiceException
    {
        super.init(serviceName, context, serviceParams);

        this.entries = new java.util.ArrayList();
        initAgents((String)getParams().get(P_AGENT_FILE));
        initAgents((String)getParams().get(P_AP_FILE));

        if (log.isDebugEnabled())
            log.debug("Service '"+getName()+"' ready for work.");

        //
        // display OpenWFE version in logs
        
        log.info("OpenWFE version : "+Definitions.OPENWFE_VERSION);
    }

    //
    // METHODS

    /**
     * Returns the list of automatic participant descriptions.
     */
    protected java.util.List getEntries ()
    {
        return this.entries;
    }

    /**
     * Loads the automatic participant descriptions as enumerated within
     * a file.
     */
    protected void initAgents (String agentFileName)
        throws ServiceException
    {
        //if (agentFileName == null)
        //{
        //    throw new ServiceException
        //        ("Parameter '"+P_AGENT_FILE+
        //         "' or '"+P_AP_FILE+
        //         "' is mandatory for service '"+getName()+"'");
        //}

        if (agentFileName == null) return;

        agentFileName = FileUtils.expandUrl
            (getContext().getApplicationDirectory(), agentFileName);

        org.jdom.Element rootElt = null;
        try
        {
            rootElt = XmlUtils.extractXml(agentFileName, false);
        }
        catch (Exception e)
        {
            throw new ServiceException
                ("Failed to build agents from "+agentFileName, e);
        }

        //
        // 'agent'

        java.util.Iterator it = rootElt.getChildren(AGENT).iterator();

        while (it.hasNext())
            initAgent((org.jdom.Element)it.next());

        //
        // 'automatic-participant'

        it = rootElt.getChildren(AP).iterator();

        while (it.hasNext())
            initAgent((org.jdom.Element)it.next());
    }

    /**
     * Builds an agent description from an XML element.
     */
    protected void initAgent (final org.jdom.Element elt)
    {
        String agentName = elt.getAttributeValue(NAME);
        String agentRegex = elt.getAttributeValue(REGEX);
        if (agentRegex != null) agentName = agentRegex;

        String agentBuilder = elt.getAttributeValue(BUILDER);
        String agentClass = elt.getAttributeValue(CLASS);

        String instantiate = elt.getAttributeValue(INSTANTIATE);
        if (instantiate == null)
            instantiate = INSTANTIATE_DEFAULT_VALUE;
        else
            instantiate = instantiate.trim().toLowerCase();

        java.util.Map params = Parameters.extractParameters(elt);

        try
        {
            Entry entry = null;
            if (instantiate.equals("once"))
            {
                Agent agent = instantiateAgent
                    (agentBuilder, agentName, agentClass, params);

                entry = new InstantiateOnceEntry(agent);
            }
            else
            {
                entry = new InstantiateEachTimeEntry
                    (agentBuilder,
                     agentName,
                     agentClass,
                     params);
            }

            this.entries.add(entry);

            log.info
                ("Prepared agent "+
                 agentName+"::"+agentBuilder+"::"+agentClass);
        }
        catch (final Throwable t)
        {
            log.error("Failed to instantiate agent. Skipping it", t);
        }
    }

    /**
     * Does the instantiation job for an agent.
     */
    protected Agent instantiateAgent 
        (final String agentBuilder, 
         final String agentName, 
         final String agentClass, 
         final java.util.Map params)
    throws 
        ApreException
    {
        try
        {
            Class agentBuilderClass =
                Class.forName(agentBuilder);
            AgentBuilder builder = 
                (AgentBuilder)agentBuilderClass.newInstance();
            Agent agent = builder
                .build(agentName, agentClass, getContext(), params);

            return agent;
        }
        catch (final ApreException re)
        {
            throw re;
        }
        catch (final Exception e)
        {
            throw new ApreException
                ("Failed to instantiate agent "+
                 agentBuilder+"::"+agentClass, e);
        }
    }

    //
    // METHODS from Router

    /**
     * Returns an agent instance corresponding to the participant name.
     */
    public Agent determineAgent (final String participantName)
        throws ApreException
    {
        java.util.Iterator it = this.entries.iterator();
        while (it.hasNext())
        {
            Entry entry = (Entry)it.next();
            if (entry.matches(participantName)) return entry.getAgent();
        }

        return null;
    }

    //
    // METHODS from Service

    /**
     * Returns the status of this router service.
     */
    public org.jdom.Element getStatus ()
    {
        final org.jdom.Element result = new org.jdom.Element(getName());

        result.addContent(XmlUtils.getClassElt(this));
        result.addContent(XmlUtils.getRevisionElt("$Id: BasicRouter.java 3145 2006-08-31 03:21:38Z jmettraux $"));

        return result;
    }

}
