/*
 * Copyright (c) 2005, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: JavaAgentBuilder.java 3145 2006-08-31 03:21:38Z jmettraux $
 */

//
// JavaAgentBuilder.java
//
// jmettraux@openwfe.org
//
// generated with 
// jtmpl 1.0.04 31.10.2002 John Mettraux (jmettraux@openwfe.org)
//

package openwfe.org.apre.impl;

import java.net.URL;
import java.net.URLClassLoader;

//import openwfe.org.Utils;
import openwfe.org.ApplicationContext;
import openwfe.org.apre.Agent;
import openwfe.org.apre.AgentBuilder;
import openwfe.org.apre.ApreException;


/**
 * Easy task : instatiating a java class...
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Date: 2006-08-31 05:21:38 +0200 (Thu, 31 Aug 2006) $
 * <br>$Id: JavaAgentBuilder.java 3145 2006-08-31 03:21:38Z jmettraux $ </font>
 *
 * @author jmettraux@openwfe.org
 */
public class JavaAgentBuilder

    implements AgentBuilder

{

    private final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(JavaAgentBuilder.class.getName());

    //
    // FIELDS

    //
    // CONSTRUCTORS

    //
    // METHODS from AgentBuilder

    public Agent build 
        (final String agentName,
         final String agentClass, 
         final ApplicationContext context, 
         final java.util.Map agentParams)
    throws 
        ApreException
    {
        try
        {
            Class clazz = null;

            final String path = (String)agentParams.get(Agent.P_AGENT_PATH);

            if (path != null)
                //
                // localize agent in its own PATH
            {
                URL url = null;
                //String fullPath = null;
                try
                {
                    /*
                    fullPath = 
                        context.getApplicationDirectory() + 
                        java.io.File.separator + 
                        path;
                    final URL url = (new java.io.File(fullPath)).toURL();
                    */

                    /*
                    url = new URL(Utils.expandUrl
                        (context.getApplicationDirectory(), path));
                    */

                    url = (new java.io.File(path)).toURL();

                    final ClassLoader loader = 
                        new URLClassLoader(new URL[] { url });

                    clazz = loader.loadClass(agentClass);
                }
                catch (final Throwable t)
                {
                    log.warn
                        ("Failed to set up classloader for path "+
                         url.toString(), 
                         t);
                }
            }

            if (clazz == null)
                //
                // else localize agent in CLASSPATH
            {
                clazz = Class.forName(agentClass);
            }

            final Agent agent = (Agent)clazz.newInstance();

            agent.init(agentName, context, agentParams);

            return agent;
        }
        catch (Exception e)
        {
            throw new ApreException
                ("Failed to build java agent", e);
        }
    }

    //
    // METHODS

}
