/*
 * Copyright (c) 2005, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: JythonAgentBuilder.java 3145 2006-08-31 03:21:38Z jmettraux $
 */

//
// JythonAgentBuilder.java
//
// jmettraux@openwfe.org
//
// generated with 
// jtmpl 1.0.04 31.10.2002 John Mettraux (jmettraux@openwfe.org)
//

package openwfe.org.apre.impl;

import openwfe.org.ApplicationContext;
import openwfe.org.apre.Agent;
import openwfe.org.apre.AgentBuilder;
import openwfe.org.apre.ApreException;
import openwfe.org.apre.jython.PythonInterpreter;


/**
 * Instantiates an agent out of a PythonInterpreter
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Date: 2006-08-31 05:21:38 +0200 (Thu, 31 Aug 2006) $
 * <br>$Id: JythonAgentBuilder.java 3145 2006-08-31 03:21:38Z jmettraux $ </font>
 *
 * @author jmettraux@openwfe.org
 */
public class JythonAgentBuilder

    implements AgentBuilder

{

    //
    // CONSTANTS & co

    public final static String AGENT_DIRECTORY
        = "agentDirectory";

    //
    // FIELDS

    //
    // CONSTRUCTORS

    //
    // METHODS from AgentBuilder

    public Agent build 
        (String agentName,
         String agentClass, 
         ApplicationContext context, 
         java.util.Map agentParams)
    throws 
        ApreException
    {
        //
        // prepare interpreter
        
        String agentDirectory = (String)agentParams.get(AGENT_DIRECTORY);

        if (agentDirectory != null)
        {
            PythonInterpreter.getInstance()
                .loadAndInterpretDirectory(agentDirectory);
        }

        //
        // instantiate agent

        Agent agent = (Agent)PythonInterpreter.getInstance()
            .instantiate(agentClass, Agent.class);

        if (agent == null)
        {
            throw new ApreException
                ("Could not instantiate agent of class "+agentClass);
        }

        agent.init(agentName, context, agentParams);

        return agent;
    }

    //
    // METHODS

}
